
path       = require 'path'
Methods    = require './methods'
methods    = new Methods
_          = require 'lodash'
yaml       = require 'yamljs'
debug      = (require 'debug')('inker')

###*
 * Singleton config class , as there is no way
 * to change templates on runtime.
###

class Config 

  ###*
   * private var to make sure we are not re-initiating
   * the same class.
   * @type {[Object]} [defaults to null]
  ###
  instance = null

  defaults =
    project_name     : 'Inker'
    public_dir       : 'public'
    output_dir       : 'docs'
    theme            : 'default'
    content_dir      : 'content'
    templates_dir    : 'templates'
    partials_dir     : 'partials'
    helpers_dir      : 'helpers'
    showdown_plugins : 'showdown'
    css_dir          : 'css'
    js_dir           : 'js'
    

  constructor: () ->
    ###*
     * Setting up instance and config 
     * only when they are not set
    ###
    if !instance 
      instance = this
      @config = {}

    return instance  

  loadConfig : (configFile) ->
  
    ###*
      setting up configFile with defaults to argumented value
      @if null
        build path expecting config.yaml exists inside root of project
      @else
        resolve argument and use it as configFile
      /if
    ###
    
    debug "loading config from #{configFile}"

    try
      config        = yaml.load configFile
      @config       = _.assign defaults,config
    catch e
      @config    = defaults
      
    @config.templates_dir = "themes/#{@config.theme}/#{@config.templates_dir}"
    @config.partials_dir  = "themes/#{@config.theme}/#{@config.partials_dir}"
    @config.helpers_dir   = "themes/#{@config.theme}/#{@config.helpers_dir}"

    @config

  getConfig : (configFile) ->
    
    if _.size @config
      return @config
    else
      @config = @loadConfig configFile
      

module.exports = Config