
path       = require 'path'
Methods    = require './methods'
methods    = new Methods
fs         = require 'fs'
_          = require 'lodash'
debug      = (require 'debug')('inker')

###*
 * Singleton templates class , as there is no way
 * to change templates on runtime.
###

class Templates 

  ###*
   * private var to make sure we are not re-initiating
   * the same class.
   * @type {[Object]} [defaults to null]
  ###
  instance = null

  constructor: () ->
    ###*
     * Setting up instance,templates and partials 
     * only when they are not set
    ###
    if !instance 
      instance = this
      @templates = {}
      @partials  = {}

    return instance


  loadHandleBarsTemplates = (entity) ->
    ###
      entities are logical entities like 
      - templates
      - partials
    ###
    filesArrayToReturn = {}

    ###*
      * Absolute path to dir
    ###
    entityPath   = entity
    debug "loading entity from #{entityPath} path"


    ###*
      * Reading directory to list all files
      * using readDir thunk , facade for fs.readdir 
    ###
    try
      filesArray  = yield methods.readDir entityPath
    catch e
      filesArray = []
    debug "filesArray from readDir #{filesArray}"

    ###*
      * Since readdir does not return relative paths, 
      * we need to join them using _ map method
    ###
    filesArray  = _.map filesArray, (file) ->
      path.join entityPath,file

    debug "filesArray after resolving paths #{filesArray}"

    ###*
      * Using asyncMap thunk to an array of files and store
      * their <Buffer> object as an array
    ###
    filesContent = yield methods.asyncMap filesArray,fs.readFile

    ###*
      finally mapping through all the files and setting them
      as key/value pairs , where
      - key is the file name
      - value is the file buffer
    ###
    _.map filesContent, (partial,i) ->
      fileName = path.basename filesArray[i]
      filesArrayToReturn[fileName] = partial

    ###*
      implict return
    ###
    return filesArrayToReturn
  

  loadTemplates : (templatesPath) ->
  
    debug "loading templates from #{templatesPath}"
            
    @templates = yield loadHandleBarsTemplates templatesPath
    

  getTemplates : (templatesPath) ->
  
    if _.size @templates
      debug "returning previously loaded templates"
      return @templates
    else
      @templates = @loadTemplates templatesPath


  loadPartials : (partialsPath) ->

    debug "loading partials from #{partialsPath}"

    @partials = yield loadHandleBarsTemplates partialsPath


  getPartials : (partialsPath) ->
  
    if _.size @partials
      debug "returning previously loaded partials"
      return @partials
    else
      @partials = @loadPartials partialsPath

module.exports = Templates