
chai      = require 'chai'
should    = chai.should()
expect    = chai.expect
assert    = chai.assert
mkdirp    = require 'mkdirp'

Config    = require '../lib/config'
config    = new Config

fs        = require 'fs'
exec      = require('child_process').exec

configText = """project_name : 'My Project'
public_dir   : 'dist'
output_dir   : 'docs'
css_dir      : 'css'
js_dir       : 'js'"""

invalidConfigText = """project_name - 'Wrong Syntax'
public_dir   : 'public'
output_dir   : 'docs'
css_dir      : 'css'
js_dir       : 'js'
theme        : 'default'"""

semiConfigText = """project_name : 'My Project'
theme : 'fancy'"""


describe "Config", () ->
  
  describe "loadConfig" , () ->

    afterEach (done) ->
      exec 'rm -rf ./tmp', () ->
        done()
    
    context "when not present", () ->
      it "should return defaults when no arguments have been passed" , () ->
        conf = config.loadConfig()
        expect(conf).to.be.an("object")

    context "when invalid", () ->

      before (done) ->
        mkdirp 'tmp', (err) ->
          fs.writeFile 'tmp/config.yaml',invalidConfigText, (error) ->
            done()

      it "should return default configuration when config file is not valid" , () ->
        conf = config.loadConfig("./tmp/config.yaml")
        expect(conf).to.be.an "object"
        expect(conf).to.have.property("project_name").with.equal "Inker"

    context "when present", () ->

      before (done) ->
        mkdirp 'tmp', (err) ->
          fs.writeFile 'tmp/config.yaml',configText, (error) ->
            done()
    
      it "should parse config file and return valid object" , (done) ->
        conf = config.loadConfig("./tmp/config.yaml")
        conf.should.be.an "object"
        conf.should.have.property("project_name").with.be.a("string")
        conf.should.have.property("theme").with.equal("default")
        done()

  describe "extendDefaults", () ->

    before (done) ->
      mkdirp 'tmp', (err) ->
        fs.writeFile 'tmp/config.yaml',semiConfigText, (error) ->
          done()

    it "should extend defaults properly" , (done) ->
      conf = config.loadConfig("./tmp/config.yaml")
      conf.should.be.an "object"
      conf.should.have.property("project_name").with.equal("My Project")
      conf.should.have.property("theme").with.equal("fancy")
      conf.should.have.property("output_dir").with.equal("docs")
      done()
  
   describe "singleTon", () ->
    context "getMenu", () ->
      it "should return previously loaded menu", () ->
        config1 = new Config
        conf = config1.getConfig()
        conf.should.be.an "object"
        conf.should.have.property("project_name").with.equal("My Project")
        conf.should.have.property("theme").with.equal("fancy")
        conf.should.have.property("output_dir").with.equal("docs")