
chai         = require 'chai'
should       = chai.should()
expect       = chai.expect
assert       = chai.assert
mkdirp       = require 'mkdirp'
fs           = require 'fs'
co           = require 'co'
exec         = require('child_process').exec

Templates    = require '../lib/templates'
templates    = new Templates
async        = require 'async'

htmlContent = "<html><body><h2>Hello inker</h2></body></html>"

describe "Template" , () ->

  describe "loadTemplates" , () ->

    afterEach (done) ->
      exec 'rm -rf ./tmp', () ->
        done()

    context "when templates are missing" , () ->
      
      it "should return empty object when templates do not exists", () ->

        tmpls = co.wrap () ->
          templates.loadTemplates("tmp/templates")
        
        tmpls()
        .then (success) ->
          should.exist success
          expect(Object.keys(success)).to.have.length 0
        
    context "when templates exists" , () ->
      
      before (done) ->
        mkdirp 'tmp/templates', (err) ->
          fs.writeFile 'tmp/templates/index.handlebars',htmlContent, (error) ->
            done()
      
      it "should return an object of templates", (done) ->
        tmpls = co.wrap () ->
          templates.loadTemplates("tmp/templates")
        
        tmpls()
        .then (success) ->
          should.exist success
          assert.isObject success
          expect(success).to.have.property("index.handlebars")
          done()
        .catch (err) ->

    context "when multiple templates", () ->
      
      writeFile = (file,cb) ->
        fs.writeFile file,htmlContent,cb
    
      before (done) ->
        mkdirp 'tmp/templates', (err) ->
          async.map ['tmp/templates/index.handlebars','tmp/templates/default.handlebars'],writeFile, () ->
            done()
            
      it "should return an object with 2 templates" , () ->
        tmpls = co.wrap () ->
          templates.loadTemplates "tmp/templates"
        
        tmpls()
        .then (success) ->
          should.exist success
          assert.isObject success
          expect(Object.keys(success)).to.have.length 2
          expect(success).to.have.property("index.handlebars")
          expect(success).to.have.property("default.handlebars")
          
  describe "getTemplates", () ->
    context "singleTon", () ->
      
      it "should return previously loaded templates" , () ->
        
        templates1 = new Templates()
        tmpls = co.wrap () ->
          templates1.getTemplates()
        
        tmpls()
        .then (success) ->
          should.exist success
          assert.isObject success
          expect(Object.keys(success)).to.have.length 2
          expect(success).to.have.property("index.handlebars")
          expect(success).to.have.property("default.handlebars")
        
          
  describe "loadPartials" , () ->

    afterEach (done) ->
      exec 'rm -rf ./tmp', () ->
        done()

    context "when partials are missing" , () ->
      
      it "should return empty object when partials do not exists", () ->

        partials = co.wrap () ->
          templates.loadPartials("tmp/partials")
        
        partials()
        .then (success) ->
          should.exist success
          expect(Object.keys(success)).to.have.length 0
        
    context "when partials exists" , () ->
      
      before (done) ->
        mkdirp 'tmp/partials', (err) ->
          fs.writeFile 'tmp/partials/header.handlebars',htmlContent, (error) ->
            done()
      
      it "should return an object of partials", (done) ->
        partials = co.wrap () ->
          templates.loadPartials("tmp/partials")
        
        partials()
        .then (success) ->
          should.exist success
          assert.isObject success
          expect(success).to.have.property("header.handlebars")
          done()
        .catch (err) ->

    context "when multiple partials", () ->
      
      writeFile = (file,cb) ->
        fs.writeFile file,htmlContent,cb
    
      before (done) ->
        mkdirp 'tmp/partials', (err) ->
          async.map ['tmp/partials/header.handlebars','tmp/partials/footer.handlebars'],writeFile, () ->
            done()
            
      it "should return an object with 2 partials" , () ->
        partials = co.wrap () ->
          templates.loadPartials "tmp/partials"
        
        partials()
        .then (success) ->
          should.exist success
          assert.isObject success
          expect(Object.keys(success)).to.have.length 2
          expect(success).to.have.property("header.handlebars")
          expect(success).to.have.property("footer.handlebars")
          
  describe "getPartials", () ->
    context "singleTon", () ->
      
      it "should return previously loaded templates" , () ->
        
        partials1 = new Templates()
        partials = co.wrap () ->
          partials1.getPartials()
        
        partials()
        .then (success) ->
          should.exist success
          assert.isObject success
          expect(Object.keys(success)).to.have.length 2
          expect(success).to.have.property("header.handlebars")
          expect(success).to.have.property("footer.handlebars")
